==> spirit_hub-0.0.1.9.tar.gz: spirit_hub-0.0.1.9/app/controllers/concerns/state_form.rb <==
# frozen_string_literal: true

module StateForm
  extend ActiveSupport::Concern

  private

  def add_action(models, params = {})
    return unless (next_state = find_next_state(params[:add_action] == 'true', params[:next_state_id]))
    return update_state_to_done(models) if next_state.key == :done

    generate_action_form(next_state.key, models)
  end

  def generate_action_form(state_key, models)
    models = Array(models)
    case state_key.to_sym
    when :classify
      generate_classify_form(models)
    when :share
      generate_share_form(models)
    when :validate, :refuse
      generate_evaluate_form(state_key, models)
    when :to_share, :to_validate, :to_complete, :to_answer
      generate_send_to_form(state_key, models)
    when :to_sign
      generate_to_sign_form(models)
    when :done
      make_tagsets_doc_done(models)
    end
  end

  def generate_classify_form(models)
    tagsets = Tagset.where(type: 'directory').order_by(updated_at: :desc).limit(5)
    state = SpiritHub::HubdocCycle.find_state_by_key(:classify)
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/classify_form', locals: { models:, tagsets:, ids: models.map(&:id).join(','), state: })
  end

  def generate_share_form(models)
    state = SpiritHub::HubdocCycle.find_state_by_key(:share)
    share_candidates = SpiritHub::TagsetShare.candidates_to_select
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/share_form', locals: { model: models.first, state:, share_candidates: })
  end

  def generate_evaluate_form(state_key, models)
    state = SpiritHub::HubdocCycle.find_state_by_key(state_key)
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/evaluate_form',
                                                  locals: { model: models.first, state:, evaluate_users: user_group_for_select })
  end

  def generate_send_to_form(state_key, models)
    state = SpiritHub::HubdocCycle.find_state_by_key(state_key)
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/send_to_form',
                                                  locals: { model: models.first, state:, send_to_user_groups: user_group_for_select })
  end

  def generate_to_sign_form(models)
    state = SpiritHub::HubdocCycle.find_state_by_key(:to_sign)
    user_candidates = SpiritHub::TagsetRequestSignature.user_candidates_to_select
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/to_sign_form', locals: { model: models.first, state:, user_candidates: })
  end

  # TODO : to change when we have the user/group/contact select search AJAX system
  def user_group_for_select
    User.all.map { |u| ["#{u.full_name} (#{u.email})", u.id] } + Group.all.map { |g| ["Groupe: #{g.name} (#{g.user_count} utilisateurs)", "g_#{g.id}"] }
  end

  def make_tagsets_doc_done(models)
    models.each do |model|
      SpiritHub::HubdocCycle.change_state(model, :done, @user)
      model.save
      update_turbo_action_current_state(model)
    end
    flash.now[:notice] = 'Documents traités'
    update_flash_message
  end

  def update_turbo_action_current_state(model)
    @turbo_actions << turbo_stream.replace("last_current_user_name_#{model.id}", partial: 'spirit_hub/common/last_current_user_name', locals: { model: })
    @turbo_actions << turbo_stream.update("tagset_#{model.id}_current_state_percent", partial: 'spirit_hub/common/current_state_percent', locals: { model: })

    hide_delete_action = route_controller_from_referer == 'spirit_hub/directories'
    @turbo_actions << turbo_stream.update("row_action_#{model.id}", partial: 'spirit_hub/common/row_actions', locals: { model:, hide_delete_action: })
  end

  def find_next_state(add_action, state_id)
    add_action && state_id ? State.find(state_id) : nil
  rescue ActiveRecord::RecordNotFound
    nil
  end

  def update_state_to_done(models, next_state_id)
    next_state_id ||= State.find_by!(key: :done).id
    Array(models).each { |model| model.update(state_id: next_state_id) }
  end
end

==> spirit_hub-0.0.1.10.tar.gz: spirit_hub-0.0.1.10/app/controllers/concerns/state_form.rb <==
# frozen_string_literal: true

module StateForm
  extend ActiveSupport::Concern

  private

  def add_action(models, params = {})
    return unless (next_state = find_next_state(params[:add_action] == 'true', params[:next_state_id]))
    return update_state_to_done(models) if next_state.key == :done

    generate_action_form(next_state.key, models)
  end

  def generate_action_form(state_key, models)
    models = Array(models)
    case state_key.to_sym
    when :classify
      generate_classify_form(models)
    when :share
      generate_share_form(models)
    when :validate, :refuse
      generate_evaluate_form(state_key, models)
    when :to_share, :to_validate, :to_complete, :to_answer
      generate_send_to_form(state_key, models)
    when :to_sign
      generate_to_sign_form(models)
    when :done
      make_tagsets_doc_done(models)
    end
  end

  def generate_classify_form(models)
    tagsets = Tagset.where(type: 'directory').order_by(updated_at: :desc).limit(5)
    state = SpiritHub::HubdocCycle.find_state_by_key(:classify)
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/classify_form', locals: { models:, tagsets:, ids: models.map(&:id).join(','), state: })
  end

  def generate_share_form(models)
    state = SpiritHub::HubdocCycle.find_state_by_key(:share)
    share_candidates = SpiritHub::ActorFormatter.actors_to_select(User.all.limit(5) + Group.all.limit(5) + Doc.where(type: 'contact').limit(5))
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/share_form', locals: { model: models.first, state:, share_candidates: })
  end

  def generate_evaluate_form(state_key, models)
    state = SpiritHub::HubdocCycle.find_state_by_key(state_key)
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/evaluate_form',
                                                  locals: { model: models.first, state:, evaluate_users: user_group_for_select })
  end

  def generate_send_to_form(state_key, models)
    state = SpiritHub::HubdocCycle.find_state_by_key(state_key)
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/send_to_form',
                                                  locals: { model: models.first, state:, send_to_user_groups: user_group_for_select })
  end

  def generate_to_sign_form(models)
    state = SpiritHub::HubdocCycle.find_state_by_key(:to_sign)
    user_candidates = SpiritHub::TagsetRequestSignature.user_candidates_to_select
    @turbo_actions << turbo_stream.update(:modal, partial: 'spirit_hub/common/state_forms/to_sign_form', locals: { model: models.first, state:, user_candidates: })
  end

  def user_group_for_select
    SpiritHub::ActorFormatter.actors_to_select(User.all.limit(5) + Group.all.limit(5))
  end

  def make_tagsets_doc_done(models)
    models.each do |model|
      SpiritHub::HubdocCycle.change_state(model, :done, @user)
      model.save
      update_turbo_action_current_state(model)
    end
    flash.now[:notice] = 'Documents traités'
    update_flash_message
  end

  def update_turbo_action_current_state(model)
    @turbo_actions << turbo_stream.replace("last_current_user_name_#{model.id}", partial: 'spirit_hub/common/last_current_user_name', locals: { model: })
    @turbo_actions << turbo_stream.update("tagset_#{model.id}_current_state_percent", partial: 'spirit_hub/common/current_state_percent', locals: { model: })
    @turbo_actions << turbo_stream.update("row_action_#{model.id}", partial: 'spirit_hub/common/row_actions', locals: { model: })
  end

  def find_next_state(add_action, state_id)
    add_action && state_id ? State.find(state_id) : nil
  rescue ActiveRecord::RecordNotFound
    nil
  end

  def update_state_to_done(models, next_state_id)
    next_state_id ||= State.find_by!(key: :done).id
    Array(models).each { |model| model.update(state_id: next_state_id) }
  end
end
